import math

def _is_scalar(value):
    """
    Primitive version, relying on the fact that JSON cannot
    contain any more complicated data structures.
    """
    return not isinstance(value, (list, tuple, dict))

def _prefix_lines(s: str, prefix: str, prefix_first_line=True) -> str:
    """
    Prefixes lines in a string str with a string prefix.
    The first line will not be prefixed if prefix_first_line is False.
    """
    if prefix == '':
        return s
    lines = s.splitlines()
    for n in range(len(lines)):
        if n != 0 or prefix_first_line:
            lines[n] = prefix + lines[n]
    return '\n'.join(lines)

def _extend_typeof(obj):
    if obj is None:
        return 'null'
    elif isinstance(obj, list):
        return 'array'
    elif isinstance(obj, dict):
        return 'object'
    else:
	    return type(obj).__name__

def _round_obj(data, precision):
    if isinstance(data, list):
        return [_round_obj(i, precision) for i in data]
    elif isinstance(data, dict):
        for k in data:
            data[k] = _round_obj(data[k], precision)
        return data
    elif isinstance(data, float) and math.isfinite(data):
        return round(data, precision)
    else:
        return data

def _get_opt(opts, key, default=False):
    if not opts:
        return default
    if isinstance(opts, dict):
        val = opts.get(key, default)
    else:
        val = getattr(opts, key, default)
    return val if val is not None else default

def _set_opt(opts, key, val):
    d = {}
    if not opts:
        return
    if isinstance(opts, dict):
        opts[key] = val
    else:
        setattr(opts, key, val)



import math
import re
import json
from abc import ABC, abstractmethod
from typing import Any

from ..util import _get_opt, _extend_typeof, _is_scalar
from ..comparator import OP

class FormatterError(ValueError):
    pass

class Part:
    BODY = 'b'
    ELISION = 'e'
    OBJECT_BEGIN = 'O'
    OBJECT_END = 'o'
    ARRAY_ELEMENT = '-'
    ARRAY_BEGIN = 'A'
    ARRAY_END = 'a'

_ansi = lambda code: '\x1b['+str(code)+'m'

Theme = {
    OP.NONE: lambda c: c,
    OP.ADD: lambda c: _ansi(32) + c +_ansi(0),
    OP.REMOVE: lambda c: _ansi(31) + c +_ansi(0),
}

class BaseFormatter(ABC):
    """Base class containing common formatter code to make writing formatters easier"""

    def __init__(self, diff = None, opts = None):
        self.diff = diff
        self.opts = opts

    def _get_opt(self, key, default=False):
        return _get_opt(self.opts, key, default)

    @abstractmethod
    def _output(self, context: Any, op: str, part: str, key: str, value: Any, depth: int):
        pass

    def _output_elisions(self, context: Any, n, depth: int):
        max_elisions = self._get_opt('max_elisions', math.inf)
        if n < max_elisions:
            for i in range(0, n):
                self._output(context, OP.NONE, Part.ELISION, '', '...', depth)
        else:
            self._output(context, OP.NONE, Part.ELISION, '', f'... ({n} entries)', depth)

    def _output_diff(self, context: Any, key: str, diff: Any, op = OP.NONE, depth = 0):
        subvalue = None
        subdepth = depth+1

        typ = _extend_typeof(diff)
        if typ == 'object':
            if ('__old' in diff) and ('__new' in diff) and (len(diff) == 2):
                if _is_scalar(diff['__old']) and _is_scalar(diff['__new']):
                    return self._output(context, OP.MODIFY, Part.BODY, key, diff, depth)
                else:
                    self._output_diff(context, key, diff['__old'], OP.REMOVE, depth)
                    return self._output_diff(context, key, diff['__new'], OP.ADD, depth)
            else:
                self._output(context, op, Part.OBJECT_BEGIN, key, None, depth)
                for subkey in diff:
                    m = None
                    subvalue = diff[subkey]
                    m = re.match(r'^(.*)__deleted$', subkey)
                    if m:
                        self._output_diff(context, m[1], subvalue, OP.REMOVE, subdepth)
                    else:
                        m = re.match(r'^(.*)__added$', subkey)
                        if m:
                            self._output_diff(context, m[1], subvalue, OP.ADD, subdepth)
                        else:
                            self._output_diff(context, subkey, subvalue, op, subdepth)
                return self._output(context, op, Part.OBJECT_END, key, None, depth)

        elif typ == 'array':
            self._output(context, op, Part.ARRAY_BEGIN, key, None, depth)
        
            looks_like_diff = True
            for item in diff:
                if (_extend_typeof(item) != 'array') or not ((len(item) == 2) or ((len(item) == 1) and (item[0] == ' '))) or not (isinstance(item[0], str)) or (len(item[0]) != 1) or item[0] not in [' ', '-', '+', '~']:
                    looks_like_diff = False
        
            if looks_like_diff:
                subop = OP.NONE
                elision_count = 0
                for it in diff:
                    subop = it[0]
                    subvalue = it[1] if len(it) > 1 else None
                    if subop == OP.NONE and subvalue is None:
                        elision_count+=1
                    else:
                        if elision_count > 0:
                            self._output_elisions(context, elision_count, subdepth)
                        elision_count = 0
            
                        if subop not in [OP.NONE, OP.MODIFY, OP.ADD, OP.REMOVE]:
                            raise FormatterError(f'Unexpected op \'{subop}\' in {json.dumps(diff, indent="  ")}')
                        
                        if subop == OP.MODIFY:
                            subop = OP.NONE

                        self._output_diff(context, '', subvalue, subop, subdepth)

                        if elision_count > 0:
                            self._output_elisions(context, elision_count, subdepth)
            else:
                for subvalue in diff:
                    self._output_diff(context, '', subvalue, op, subdepth)
        
            return self._output(context, op, Part.ARRAY_END, key, None, depth)
        else:
            if diff == 0 or diff is None or diff == False or diff == '' or diff:
                return self._output(context, op, Part.BODY, key, diff, depth)

    def stringify(self, diff = None, opts = None):
        """
        Produces a human-readable diff text lines from a dict of differences created by Comparator.
        """
        if diff is None:
            diff = self.diff
        if opts is None:
            opts = self.opts

        if diff is None:
            return ''

        output = []

        def output_cb(op, line):
            nonlocal output

            line = f'{op}{line}'
            if self._get_opt('color', False):
                theme = self._get_opt('theme', Theme)
                if op in theme:
                    line = theme[op](line)
            output.append(line)

        self._output_diff({'output': output_cb}, '', diff)

        return '\n'.join(output)

    def __str__(self):
        return self.stringify()



#!/usr/bin/env python3

import json
import sys
from argparse import ArgumentParser

from .comparator import diff
from .formatters import colorize, YAMLFormatter

def main(argv=None):
    parser = ArgumentParser(prog='struct_diff')
    parser.add_argument('old', help='original file')
    parser.add_argument('new', help='new file')
    parser.add_argument('-C', dest='color', default=None, action='store_true', help='force colorize the output')
    parser.add_argument('--no-color', action='store_true', help='do not colorize the output')
    parser.add_argument('-j', '--raw-json', action='store_true', help='display raw JSON encoding of the diff')
    parser.add_argument('-Y', '--yaml', action='store_true', help='output diff as YAML')
    parser.add_argument('-f', '--full', action='store_true', help='include the equal sections of the document, not just the deltas')
    parser.add_argument('--max-elisions', type=int, help='max number of ...\'s to show in a row in "deltas" mode (before collapsing them) #var(maxElisions)')
    parser.add_argument('-o', '--output-keys', metavar='KEY', nargs='+', help='always print this comma separated keys, with their value, if they are part of an object with any diff')
    parser.add_argument('-n', '--output-new-only', action='store_true', help='output only the updated and new key/value pairs (without marking them as such). If you need only the diffs from the old file, just exchange the first and second json')
    parser.add_argument('-s', '--sort', action='store_true', help='sort primitive values in arrays before comparing')
    parser.add_argument('-c', '--object-context', action='store_true', help='if a scalar value of an object key is changed, also include other (unchanged) values of that object')
    parser.add_argument('-k', '--keys-only', action='store_true', help='compare only the keys, ignore the differences in values')
    parser.add_argument('-K', '--keep-unchanged-values', action='store_true', help='instead of omitting values that are equal, output them as they are')
    parser.add_argument('-p', '--precision', metavar='DECIMALS', type=int, help='round all floating point numbers to this number of decimal places prior to comparison')
    parser.add_argument('-w', '--indent-width', default=None, type=int, help='number of spaces for indendation')

    sys_args = argv if argv is not None else sys.argv[:]
    args = parser.parse_args()

    if args.no_color:
        args.color = False
    else:
        if args.color is None:
            args.color = sys.stdout.isatty()

    with open(args.old) as old_file, open(args.new) as new_file:
        try:
            obj1 = json.load(old_file)
        except Exception as e:
            print(f"error parsing file {args.old} as JSON: {e}", file=sys.stderr)
            old_file.seek(0, 0)
            obj1 = old_file.read()
        
        try:
            obj2 = json.load(new_file)
        except Exception as e:
            print(f"error parsing file {args.old} as JSON: {e}", file=sys.stderr)
            new_file.seek(0, 0)
            obj2 = new_file.read()

        diff_res = diff(obj1, obj2, args)

        if args.yaml:
            outs = str(YAMLFormatter(diff_res, args))
        elif args.raw_json:
            outs = json.dumps(diff_res, indent=2, ensure_ascii=False)
        else:
            outs = str(colorize(diff_res, args))

        print(outs, end=None if len(outs)>0 else '')

    # return 1 if there were differences
    if diff_res is not None and len(diff_res) > 0:
        return 1

    return 0

if __name__ == "__main__":
    sys.exit(main())



from difflib import SequenceMatcher
import json
from enum import Enum

from .util import _extend_typeof, _round_obj, _get_opt

class OP:
    NONE = ' '
    ADD = '+'
    REMOVE = '-'
    MODIFY = '~'

class ParserError(ValueError):
    pass

class Comparator(object):
    def __init__(self, opts=None):
        self.opts = opts

    def _get_opt(self, key, default=False):
        return _get_opt(self.opts, key, default)

    def __is_scalar(self, obj):
        return not isinstance(obj, (list, dict)) or obj is None

    def object_diff(self, obj1, obj2):
        """ Compare two dicts and return {score, equal, result} dict """
        result = {}
        score = 0
        equal = True

        for key, value in obj1.items():
            if not self._get_opt('output_new_only'):
                postfix = '__deleted'
            
                if key not in obj2:
                    result[f'{key}{postfix}'] = value
                    score -= 30
                    equal = False

        for key, value in obj2.items():
            postfix = '__added' if not self._get_opt('output_new_only') else ''

            if key not in obj1:
                result[f'{key}{postfix}'] = value
                score -= 30
                equal = False

        scalars_changed = False
        for key, value1 in obj1.items():
            if key in obj2:
                score += 20
                value2 = obj2[key]
                change = self.diff(value1, value2)
                if not change['equal']:
                    result[key] = change['result']
                    equal = False
                    if self.__is_scalar(value2):
                        scalars_changed = True
                elif self._get_opt('full') or key in self._get_opt('output_keys', []):
                    result[key] = value1
                    score += min(20, max(-10, change['score'] / 5)) # BATMAN!

        # include sigling keys of an object with diffs
        if scalars_changed and self._get_opt('object_context'):
            for key, value1 in obj1.items():
                if key not in result:
                    result[key] = value1
                    score += min(20, max(-10, change['score'] / 5)) # DOUBLE BATMAN!

        if equal:
            score = 100 * max(len(obj1), 0.5)
            if not self._get_opt('full'):
                result = None
        else:
            score = max(0, score)   
        return { 'score': score, 'result': result, 'equal': equal }

    def _find_matching_object(self, item, index, fuzzy_originals):
        best_match = None

        for key, it in fuzzy_originals.items():
            if key != '__next':
                candidate = it['item']
                match_index = it['index']
                index_distance = abs(match_index - index)
                if _extend_typeof(item) == _extend_typeof(candidate):
                    score = self.diff(item, candidate)['score']
                    if not best_match or \
                        score > best_match['score'] or \
                        (score == best_match['score'] and
                        index_distance < best_match['index_distance']):
                            best_match = { 'score': score, 'key': key, 'index_distance': index_distance }

        return best_match

    def _scalarize(self, array, originals, fuzzy_originals=False):
        fuzzy_matches = {}
        if fuzzy_originals:
            # Find best fuzzy match for each object in the array
            key_scores = {}
            for index in range(0, len(array)):
                item = array[index]
                if self.__is_scalar(item):
                    continue
                best_match = self._find_matching_object(item, index, fuzzy_originals)
                best_match_key = best_match['key'] if best_match else None
                if best_match and (best_match_key not in key_scores or best_match['score'] > key_scores[best_match_key]['score']):
                    key_scores[best_match_key] = { 'score': best_match['score'], 'index': index }
            for key, match in key_scores.items():
                fuzzy_matches[match['index']] = key
    
        result = []
        for index in range(0, len(array)):
            item = array[index]
            if self.__is_scalar(item):
                result.append(item)
            else:
                def incr_return_old(d, key):
                    old = d[key]
                    d[key] = old+1
                    return old
                key = index in fuzzy_matches and fuzzy_matches[index] or '__$!SCALAR' + str(incr_return_old(originals, '__next'))
                originals[key] = { 'item': item, 'index': index }
                result.append(key)
        return result

    def _is_scalarized (self, item, originals):
        return isinstance(item, str) and item in originals

    def _descalarize(self, item, originals):
        if self._is_scalarized(item, originals):
            return originals[item]['item']
        else:
            return item

    def array_diff(self, obj1, obj2):
        """ Compare two arrays and return {score, equal, result} dict """
        originals1 = { '__next': 1 }
        seq1 = self._scalarize(obj1, originals1)
        originals2 = { '__next': originals1['__next'] }
        seq2 = self._scalarize(obj2, originals2, originals1)
    
        if self._get_opt('sort'):
            def mixd(num):
                try:
                    el = int(num)
                    return (0, el)
                except:
                    return (1, num)
            # json_diff in JS sorts alphanumerically (key=str)
            # it should probably use a mixed method (key=mixd) instead
            seq1.sort(key=mixd)
            seq2.sort(key=mixd)

        opcodes = SequenceMatcher(None, seq1, seq2).get_opcodes()
    
        result = []
        score = 0
        equal = True
    
        for op, i1, i2, j1, j2 in opcodes:
            i, j = (0, 0)
            asc, end = (0, 0)
            asc1, end1 = (0, 0)
            asc2, end2 = (0, 0)
            if not (op == 'equal' or (self._get_opt('keys_only') and op == 'replace')):
                equal = False
        
            if op == 'equal':
                end = i2
                asc = i1 <= end
                for i in range(i1, end, 1 if asc else -1):
                    item = seq1[i]
                    if self._is_scalarized(item, originals1):
                        if not self._is_scalarized(item, originals2):
                            raise ParserError(f'internal bug: is_scalarized(item, originals1) != is_scalarized(item, originals2) for item {json.dumps(item, indent=2)}')
                        item1 = self._descalarize(item, originals1)
                        item2 = self._descalarize(item, originals2)
                        change = self.diff(item1, item2)
                        if not change['equal']:
                            result.append([OP.MODIFY, change['result']])
                            equal = False
                        else:
                            if self._get_opt('full') or self._get_opt('keep_unchanged_values'):
                                result.append([OP.NONE, item1])
                            else:
                                result.append([OP.NONE])
                    else:
                        if self._get_opt('full') or self._get_opt('keep_unchanged_values'):
                            result.append([OP.NONE, item])
                        else:
                            result.append([OP.NONE])
                    score += 10
            elif op == 'delete':
                end1 = i2
                asc1 = i1 <= end1
                for i in range(i1, end1, 1 if asc1 else -1):
                    result.append([OP.REMOVE, self._descalarize(seq1[i], originals1)])
                    score -= 5
            elif op == 'insert':
                end2 = j2
                asc2 = j1 <= end2
                for j in range(j1, end2, 1 if asc2 else -1):
                    result.append([OP.ADD, self._descalarize(seq2[j], originals2)])
                    score -= 5
            elif op == 'replace':
                if not self._get_opt('keys_only'):
                    asc3, end3 = (0, 0)
                    asc4, end4 = (0, 0)
                    end3 = i2
                    asc3 = i1 <= end3
                    for i in range(i1, end3, 1 if asc3 else -1):
                        result.append([OP.REMOVE, self._descalarize(seq1[i], originals1)])
                        score -= 5
                    end4 = j2
                    asc4 = j1 <= end4
                    for j in range(i1, end4, 1 if asc4 else -1):
                        result.append([OP.ADD, self._descalarize(seq2[j], originals2)])
                        score -= 5
                else:
                    asc5, end5 = (0, 0)
                    end5 = i2
                    asc5 = i1 <= end5
                    for i in range(i1, end5, 1 if asc5 else -1):
                        change = self.diff(
                            self._descalarize(seq1[i], originals1),
                            self._descalarize(seq2[i - i1 + j1], originals2)
                        )
                        if not change['equal']:
                            result.append([OP.MODIFY, change['result']])
                            equal = False
                        else:
                            result.append([OP.NONE])
        
        if equal or len(opcodes) == 0:
            if not self._get_opt('full'):
                result = None
            else:
                result = obj1
            score = 100
        else:
          score = max(0, score)
    
        return { 'score': score, 'result': result, 'equal': equal }

    def diff(self, obj1, obj2):
        """ Compare two objects of any type and return a dict with differences """
        type1 = _extend_typeof(obj1)
        type2 = _extend_typeof(obj2)
    
        if type1 == type2:
            if type1 == 'object':
                return self.object_diff(obj1, obj2)
            elif type1 == 'array':
                return self.array_diff(obj1, obj2)
    
        # Compare primitives or complex objects of different types
        score = 100
        result = obj1
        equal = False
        if not self._get_opt('keys_only', False):
            equal = obj1 == obj2
            if not equal:
                score = 0
        
                if self._get_opt('output_new_only', False):
                    result = obj2
                else:
                    result = { '__old': obj1, '__new': obj2 }
            elif not self._get_opt('full', False):
                result = None
        else:
            equal = True
            result = None
    
        return { 'score': score, 'result': result, 'equal': equal }

def diff(obj1, obj2, opts = None):
    """
    Compare two objects and return a dict with differences
    """
    p = _get_opt(opts, 'precision', None)
    if p is not None:
        obj1 = _round_obj(obj1, p)
        obj2 = _round_obj(obj2, p)
    return Comparator(opts).diff(obj1, obj2)['result']



import json
from typing import Any

from ..comparator import OP
from .base import BaseFormatter, Part

class JSONFormatter(BaseFormatter):
    def __init__(self, diff = None, opts = None):
        super().__init__(diff, opts)

    def _output(self, context: Any, op: str, part: str, key: str, value: Any, depth: int):
        if op == OP.MODIFY:
            # split modify into two outputs for removal and add
            self._output(context, OP.REMOVE, part, key, value['__old'], depth)
            self._output(context, OP.ADD, part, key, value['__new'], depth)
            return

        indent_str = ' '*self._get_opt('indent_width', 2)
        indent = indent_str*depth
        prefix = f'{key}: ' if key else ''

        output = context['output']

        if part == Part.OBJECT_BEGIN:
            output(op, indent + prefix + '{')
        elif part == Part.OBJECT_END:
            output(op, indent + '}')
        elif part == Part.ARRAY_BEGIN:
            output(op, indent + prefix + '[')
        elif part == Part.ARRAY_END:
            output(op, indent + ']')
        elif part == Part.ELISION:
            output(op, indent + value)
        else:
            #print(f"op {op} part {part} key {key} value {value} depth {depth}")
            output(op, indent + prefix + json.dumps(value))



import difflib
import re
from typing import Any

from ..comparator import OP
from .base import BaseFormatter, Part

class YAMLFormatter(BaseFormatter):
    def __init__(self, diff = None, opts = None):
        # https://yaml.org/spec/1.2.2/#plain-style
        self.re_unsafe_str = re.compile(r'^([,\[\]{}#&*!|>\'"%@`\s]|[-?:]\s)')
        super().__init__(diff, opts)

    def _text_diff(self, prev_val: str, cur_val: str, indent_str: str, depth: int) -> list[str]:
        """Creates a simplified unified diff and returns its string representation"""
        prev_lines = (prev_val+'\n').splitlines(keepends=True)
        cur_lines = (cur_val+'\n').splitlines(keepends=True)
        d = list(difflib.unified_diff(prev_lines, cur_lines, n=1))
        if len(d) > 3:
            # strip header
            d = d[3:]
        if depth>0:
            # append depth
            for n in range(len(d)):
                l = d[n]
                if len(l) > 0:
                    d[n] = l[0] + indent_str*depth + l[1:]
        return d[:-1]

    def _format_scalar(self, val: Any) -> str:
        if isinstance(val, int):
            return str(val)
        elif isinstance(val, float):
            prec = self._get_opt('precision', None)
            return str(round(val, prec)) if prec is not None else str(val)
        else:
            s = str(val)
            if self.re_unsafe_str.match(s):
                s = "'" + s + "'"
            return s

    def _output(self, context: dict, op: str, part: str, key: str, value: Any, depth: int):
        if op == OP.MODIFY:
            old = value['__old']
            new = value['__new']
            # split non-multiline modify into two outputs for removal and add
            if not (isinstance(old, str) and isinstance(new, str) and '\n' in new.rstrip()):
                self._output(context, OP.REMOVE, part, key, value['__old'], depth)
                self._output(context, OP.ADD, part, key, value['__new'], depth)
                return

        indent_str = ' '*self._get_opt('indent_width', 1)
        indent = indent_str*depth
        prefix = f'{key}: ' if key else ''

        stack = context.get('stack', [])
        key_in_current_stack_object = context.get('key_in_current_stack_object', 0)
        output = context['output']
        
        if part == Part.OBJECT_BEGIN:
            stack.append('object')
            key_in_current_stack_object = 0
            if key != '':
                output(op, indent + prefix)
        elif part == Part.OBJECT_END:
            stack.pop()
        elif part == Part.ARRAY_BEGIN:
            stack.append('array')
            if key != '':
                output(op, indent + prefix)
        elif part == Part.ARRAY_END:
            stack.pop()
        elif part == Part.ELISION:
            output(op, indent + value)
        else:
            # array element
            if len(stack) > 0 and stack[-1] == 'array':
                prefix += '- '
            if len(stack) > 1 and stack[-1] == 'object' and stack[-2] == 'array':
                indent = indent[0:-2]
                if key_in_current_stack_object == 0:
                    prefix = f'- {key}: '
                else:
                    indent += indent_str
            # object key
            if len(stack) > 0 and stack[-1] == 'object' and key != '':
                key_in_current_stack_object += 1

            if op == OP.MODIFY:
                # the second check for OP_MODIFY, now it is surely multi-line
                diff_lines = self._text_diff(old, new, indent_str, depth+1)
                if depth > 0:
                    # print YAML multiline string header
                    output(OP.NONE, indent + prefix + '|-')
                for dl in diff_lines:
                    op = OP.NONE
                    if dl[0] == '+':
                        op = OP.ADD
                    elif dl[0] == '-':
                        op = OP.REMOVE
                    output(op, dl[1:-1])
                return

            #print(f"op {op} part {part} key {key} value {value} depth {depth}")
            output(op, indent + prefix + self._format_scalar(value))

        context['stack'] = stack
        context['key_in_current_stack_object'] = key_in_current_stack_object


from setuptools import setup, find_packages

version_string = __import__('struct_diff').__version__

setup(
    name='struct-diff',
    version=version_string,
    description='Structural comparison of two objects.',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/k3a/struct_diff',
    author='K3A',
    license='MIT',
    clasifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3"
    ],
    package_dir={'struct_diff': 'struct_diff', 'tests': 'tests'},
    packages=find_packages(),
)



__all__ = ['FormatterError', 'BaseFormatter', 'colorize',
            'JSONFormatter', 'YAMLFormatter']

from .base import *
from .json import *
from .yaml import *
from ..util import _get_opt, _set_opt

def colorize(diff, opts = None):
    """
    Produces structural diff text with ANSI escape sequences for colored output.
    It is an utility function to keep the API compatibility with json_diff JavaScript library.
    """
    if not opts or _get_opt(opts, 'color', None) is None:
        _set_opt(opts, 'color', True)
    return str(JSONFormatter(diff, opts))


"""
Structural comparison of two objects.
Based on json-diff javascript package (https://github.com/andreyvit/json-diff)
"""

__version__ = "0.9.0-6"
__copyright__ = """
    Copyright (c) 2015 Andrey Tarantsov
    Copyright (c) 2022 Mario Hros (K3A.me)
"""
__credits__ = ["Andrey Tarantsov", "Mario Hros"]

from .comparator import Comparator, diff
from .formatters import *
